import scrapy
import json
from datetime import datetime, date, timedelta

#scrapy crawl noticias -t json --nolog -o noticias.json -a year=2017 -a month=3 -a day=22

import re


TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class NoticiasItem(scrapy.Item):
    title = scrapy.Field()
    text = scrapy.Field()
    date = scrapy.Field()
    location = scrapy.Field()
    author = scrapy.Field()
    topic = scrapy.Field()
    url = scrapy.Field()


class QuotesSpider(scrapy.Spider):
	name = "noticias"
	def start_requests(self):
		# self.found = False
		# self.flag = False
		self.year = getattr(self, 'year', None)
		self.month = getattr(self, 'month', None)
		self.day = getattr(self, 'day', None)
		self.req_date = date(int(self.year), int(self.month), int(self.day))
		self.date_format = "%Y-%m-%d"
		self.baseURL = 'https://www.lajornadamaya.mx'
		section_list = ['yucatan', 'quintana-roo', 'campeche', 'deportes', 'nacional',
						'internacional', 'opinion']
		# section_list = ['deportes']

		for section in section_list:
			self.section = section
			for count in range(0,2):
				if ( count == 0 ):
					yield scrapy.Request(url=self.baseURL+'/'+section, callback=self.parse_2)
				elif (count == 1):
					# self.section = section
					self.page = 0
					self.flag = False
					self.found = False
					page = -1
					if not ( section == 'opinion' ):
						while True:
							if ( self.flag ):
								self.flag = False
								break
							page+=1
							yield scrapy.Request(url=self.baseURL+'/'+section+'?p='+str(page), callback=self.parse)
						
						if ( self.found ):
							self.found = False
							self.page -= 1
							if ( self.page > 0 ):
								self.page -= 1
							
							for pag in range(self.page, self.page+6):
								yield scrapy.Request(url=self.baseURL+'/'+section+'?p='+str(pag), callback=self.parse_page, dont_filter=True)

					else:
						yield scrapy.Request(url=self.baseURL+'/notas?opinion', callback=self.parse_page)
				
	def parse_2(self, response): # para las primeras noticias
		path_list = ['//h1[@class="title"]/a/@href', '//h2[@class="title"]/a/@href']
		link_list = []
		for path in path_list:
			link_list += response.xpath(path).extract()
		
		for link in link_list:
			if ( link[:link.rfind('/')] == self.year+'-'+self.month.zfill(2)+'-'+self.day.zfill(2) ):
				item = NoticiasItem()
				item['date'] = link[:link.rfind('/')]
				item['topic'] = response.url[response.url.rfind('/')+1:].title()
				# yield scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_item_2)
				request = scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_item_2)
				request.meta['item'] = item
				yield request


	def parse(self, response): # para los json
		json_response = json.loads(response.text)

		if not ( response.url[response.url.rfind('/')+1:] == 'notas?opinion' ):
			json_list = json_response
		else:
			json_list = json_response['articles']
		
		for line in json_list:
			this_date = datetime.strptime(line['publishDate'][:line['publishDate'].rfind(' ')], self.date_format)
			this_date = this_date.date()

			if ( this_date == self.req_date ):
				self.page = int(response.url[response.url.rfind('=')+1:])
				self.found = True
				self.flag = True
				break
			elif ( this_date < self.req_date ):
				self.flag = True
				break
				
	def parse_item_2(self, response): # para las primeras noticias
		item = response.meta['item']
		# item = NoticiasItem()
		text = ''
		# item['date'] = response.url[:response.url.rfind('/')][response.url[:response.url.rfind('/')].rfind('/')+1:]
		# item['topic'] = self.section.title()
		item['title'] = response.xpath('//article/h1/text()').extract_first()
		for paragraph in response.xpath('//*[@class="txt"]').extract():
			text += remove_tags(paragraph) + '\n'
		item['text'] = text
		item['url'] = response.url
		print item['title']
		yield item


	def parse_page(self, response): # para los json
		json_response = json.loads(response.text)

		if not ( response.url[response.url.rfind('/')+1:] == 'notas?opinion' ):
			topic = response.url[response.url.rfind('/')+1:response.url.rfind('=')-2].title()
			json_list = json_response
		else:
			json_list = json_response['articles']
			topic = 'Opinion'
		
		for line in json_list:
			this_date = datetime.strptime(line['publishDate'][:line['publishDate'].rfind(' ')], self.date_format)
			this_date = this_date.date()
			
			if ( this_date == self.req_date ):
				item = NoticiasItem()
				item['date'] = line['publishDate']
				item['topic'] = topic
				item['title'] = line['name']
				if not ( response.url[response.url.rfind('/')+1:] == 'notas?opinion' ):
					request = scrapy.Request(url=self.baseURL+line['url'], callback=self.parse_item)
				else:
					request = scrapy.Request(url=self.baseURL+'/'+line['publishDate'][:line['publishDate'].rfind(' ')]+'/'+line['uriComponent'], callback=self.parse_item)
				request.meta['item'] = item
				yield request


	def parse_item(self, response): # para los json
		item = response.meta['item']
		text = ''
		for paragraph in response.xpath('//*[@class="txt"]').extract():
			text += remove_tags(paragraph) + '\n'
		item['text'] = text
		item['url'] = response.url
		print item['title']
		yield item

